; messbox: Show a message box using the Windows API.
; Usage: messbox <message>
;        <message>	The message to display. Use quotes if it has spaces in it.
;
; Electroduck		12 October 2017		v1.00

; Copyright 2018 Electroduck
;
; Redistribution and use in source and binary forms, with or without
; modification, are permitted provided that the following conditions are met:
;
; 1. Redistributions of source code must retain the above copyright notice,
; this list of conditions and the following disclaimer.
;
; 2. Redistributions in binary form must reproduce the above copyright notice,
; this list of conditions and the following disclaimer in the documentation
; and/or other materials provided with the distribution.
;
; 3. Neither the name of the copyright holder nor the names of its contributors
; may be used to endorse or promote products derived from this software without
; specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
; AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
; ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
; LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
; SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
; INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
; CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
; ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
; POSSIBILITY OF SUCH DAMAGE.

EXTERN _MessageBoxW@16
EXTERN _GetCommandLineW@0
EXTERN _CommandLineToArgvW@8
GLOBAL _NoCRTMain@0

; Note: All strings must be defined in this fashion to be wide strings
;       Otherwise, they will display as Chinese characters!
SECTION .data
s_title:	DW 'M', 'e', 's', 's', 'a', 'g', 'e', 0
s_error:	DW 'U', 's', 'a', 'g', 'e', ':', ' ', 'm', 'e', 's', 's', 'b', 'o', 'x', ' ', '<', 'm', 's', 'g', '>', 0
mbicon:		DD 0

SECTION .bss
argc:		RESD 1

SECTION .text
; No CRT means this is called instead of WinMain (honestly we could use any entry point)
_NoCRTMain@0:
	PUSH	EBP
	MOV	EBP, ESP
	
	; __stdcall GetCommandLineW()
	CALL	_GetCommandLineW@0			; result in EAX
	
	; __stdcall CommandLineToArgvW(EAX, &argc)
	PUSH	argc
	PUSH	EAX
	CALL	_CommandLineToArgvW@8			; argv in EAX
	
	; Check if argc > 1, which means we have a message to display
	MOV	ECX, [argc]
	CMP	ECX, 1
	JG	.argcOK
	; If argc isn't > 1, we don't have enough arguments
	MOV	EAX, s_error				; usage message
	MOV	DWORD [mbicon], 10h			; error icon
	JMP	.showMsg
	
	; We are OK if argc > 1
	.argcOK:
	MOV	EAX, [EAX + 4]				; EAX = argv[1]
	
	.showMsg:
	; __stdcall MessageBoxW(0, EAX, s_title, 0)
	PUSH	DWORD [mbicon]
	PUSH	s_title
	PUSH	EAX
	PUSH	0
	CALL	_MessageBoxW@16
	
	PUSH	0
	CALL	_ExitProcess@4
	
	POP	EBP
	XOR	EAX, EAX				; a funny way of doing MOV EAX, 0
	RET